/** Setup avr libc for stdio on the serial port **/
#include <avr/io.h>
#include <stdio.h>

#define FOSC 16000000 // Clock Speed
#define BAUD 57600
#define MYUBRR FOSC/16/BAUD-1

int UsartGetC(FILE *f);
int UsartPutC(char data, FILE *f);

static FILE mystdin = FDEV_SETUP_STREAM(NULL, UsartGetC, _FDEV_SETUP_READ);
static FILE mystdout = FDEV_SETUP_STREAM(UsartPutC, NULL, _FDEV_SETUP_WRITE);

void UsartInit( unsigned int ubrr)
{
  /*Set baud rate */
  UBRR0H = (unsigned char)(ubrr>>8);
  UBRR0L = (unsigned char)ubrr;
  /* Enable receiver and transmitter */
    UCSR0B = (1<<RXEN0)|(1<<TXEN0);
  /* Set frame format: 8data, 2stop bit */
  UCSR0C = (1<<USBS0)|(3<<UCSZ00);
}

/** Blocking Get **/
int UsartGetC(FILE *f) {
  /* Wait for data to be received */
  while ( !(UCSR0A & (1<<RXC0)) ) 
    ;
  /* Get and return received data from buffer */
  return UDR0; 
}

int UsartPutC(char data, FILE *f) {
  /* Wait for empty transmit buffer */
  while ( !( UCSR0A & (1<<UDRE0)) )
    ;
  /* Put data into buffer, sends the data */
  UDR0 = data;
  if (data == '\n') // add CR to LF
    UsartPutC('\r', f);
  return 0;
}

void StdioInit() {
  UsartInit(MYUBRR);
  // fdevopen uses malloc(), which we avoid
  //fdevopen(0x0, &UsartGetC); // stdin
  //fdevopen(&UsartPutC, 0x0); // stdout and stderr
  stdin = &mystdin;
  stdout = &mystdout;
  stderr = &mystdout;
}

