#ifndef TERMINAL_HPP_
#define TERMINAL_HPP_ 1

#include <stdint.h>

#define TERM_WIDTH 40
#define TERM_HEIGHT 14

/* Circular buffer */
extern unsigned char termBuffer[TERM_HEIGHT][TERM_WIDTH];
/* Attributes should be bit-packed **/
extern unsigned char termAttrib[TERM_HEIGHT][TERM_WIDTH];
/* Circular buffer origin - allows for easy scrolling */
extern int termOffsetX;
extern int termOffsetY;
/* cursor location */
extern int cursorX;
extern int cursorY;
extern uint8_t cursorAttr; // cursor attributes
/* Attributes */
typedef enum { 
  BoldFlag = 0x1,
  InvertFlag = 0x2
} TermFlags;

// A proper modulus operator that works on negatives
#define mod(x, y) ((y + (x % y)) % y)
#define getTermBuffer(y,x) termBuffer[mod(y + termOffsetY, TERM_HEIGHT)][mod(x + termOffsetX, TERM_WIDTH)]
#define getAttrib(y,x) termAttrib[mod(y + termOffsetY, TERM_HEIGHT)][mod(x + termOffsetX, TERM_WIDTH)]


/** This is the "Terminal" interface, which provides basic
 * character terminal functionality **/
void TerminalInit();
void Put(char c);
void Puts(const char *s);
void SetBright();
void UnsetBright();
void SetInvert();
void UnsetInvert();
void SetNormal();
void SetPos(int x, int y);
void Clear();

#endif // TERMINAL_HPP_
