#include <avr/io.h>
#include "VideoUzeAsm.h"

#include "../uCmorse/morse_asm.h"

.global TIMER1_COMPA_vect
.global run_timetable

#define LINE_WIDTH 220

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Font and Terminal Globals
#define TERM_WIDTH 40
#define TERM_HEIGHT 14

#define FONT_TILESET_WIDTH 8
#define FONT_TILESET_HEIGHT 16
.global font_bitpacked

.global termBuffer ; uint8_t 2d array
.global termAttrib ; uint8_t 2d array
.global termOffsetX ;uint16_t
.global termOffsetY ;uint16_t

.section .bss
; Bit-packed current line raster
current_line0:
  .fill LINE_WIDTH/8
; Bit-packed next line raster
next_line0:
  .fill LINE_WIDTH/8

#ifndef F_CPU
#  warning F_CPU NOT DEFINED DEFAULTING TO 28 MHZ
#  define F_CPU 28636360
#endif
#define F_CPU_MS (F_CPU/1000)

#define nsec2cyc(n) ((n*F_CPU_MS)/1000000)

#define T_HSYNC  nsec2cyc(4700)
#define T_FPORCH nsec2cyc(1500)
#define T_BPORCH nsec2cyc(4700)
#define T_VIDEO  nsec2cyc(52600)
#define T_LINE (T_FPORCH + T_BPORCH + T_HSYNC + T_VIDEO)
#define T_HALF_LINE (T_LINE/2)

//define FIX1 nsec2cyc(1600)
#define FIX1 nsec2cyc(800)
//define FIX2 nsec2cyc(3600)
//define FIX1 (0)
#define FIX2 (135)

.section .data

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Time Table
;; DELAY, FUNCTION, ARG1, ARG2
timetable:
;
; FIELD 0
.word 0                               , pm(debug_trigger_toggle) $ .byte 0, 0

; pre-eq phase
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 6
; vert sync/eq pulses
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_low)    $ .byte 0, 0
.word T_HSYNC - FIX1                  , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 6
; post-eq pulses
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 6
; VBI????  I'm 10 lines short from 525 lines with out this
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_VIDEO + T_FPORCH+T_BPORCH-FIX2, pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2,10
; after this, normal field1 hsync lines start
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_BPORCH - FIX1                 , pm(sync_high)   $ .byte 0, 0
.word T_VIDEO + T_FPORCH - FIX2       , pm(render_line) $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 3, 253
; half line
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_HALF_LINE - T_HSYNC - FIX2, pm(sync_high)   $ .byte 0, 0
; finished field
;
.word 0                               , pm(restart_timetable) $ .byte 0, 0

;;; FIXME: if you want two fields and "correct" NTSC
#if 0
; FIELD 1
; pre-eq phase
.word 0                               , pm(next_frame)  $ .byte 0, 0
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 6
; vert sync/eq pulses
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_low)    $ .byte 0, 0
.word T_HSYNC - FIX1                  , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 6
; post-eq pulses - 7 in this field
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_HALF_LINE - T_HSYNC - FIX2    , pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2, 7

; VBI????  I'm 10 lines short from 525 lines with out this
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_VIDEO + T_FPORCH+T_BPORCH-FIX2, pm(sync_high)   $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 2,10

; after this, normal field1 hsync lines start
.word T_HSYNC - FIX1                  , pm(sync_low)    $ .byte 0, 0
.word T_BPORCH - FIX1                 , pm(sync_high)   $ .byte 0, 0
.word T_VIDEO + T_FPORCH - FIX1       , pm(render_line) $ .byte 0, 0
.word 0                               , pm(counted_loop)$ .byte 3, 253
; All done!  Restart.
.word 0                               , pm(restart_timetable) $ .byte 0, 0
#endif

.word 0x0200          , pm(sync_high)    $ .byte 0, 0
.word 0x0100          , pm(sync_low)    $ .byte 0, 0
.word 0x0200          , pm(sync_high)    $ .byte 0, 0
.word 0x0100          , pm(sync_low)    $ .byte 0, 0
.word 0x0200          , pm(sync_high)    $ .byte 0, 0
.word 0x0100          , pm(sync_low)    $ .byte 0, 0
;.word 0                         , pm(counted_loop)$ .byte 2, 2
.word 0                         , pm(restart_timetable) $ .byte 0, 0


;; DEBUG TIMETABLE
.word 0x0640          , pm(sync_toggle)    $ .byte 0, 0
.word 0x1000          , pm(sync_toggle)    $ .byte 0, 0
.word 0                         , pm(counted_loop)$ .byte 2, 3
.word 0x2000          , pm(sync_toggle)    $ .byte 0, 0
.word 0x3000          , pm(sync_toggle)   $ .byte 0, 0
.word 0                         , pm(counted_loop)$ .byte 2, 2
.word 0x4000          , pm(sync_toggle)    $ .byte 0, 0
.word 0                         , pm(counted_loop)$ .byte 1, 2
.word 0x5000          , pm(sync_toggle)   $ .byte 0, 0
.word 0                         , pm(counted_loop)$ .byte 1, 2
.word 0               , pm(restart_timetable) $ .byte 0, 0
;;;;






;;; State for interrupt-driven timetable func
tt_cur_ptr: ; Ptr to current timetable entry
.dc.w timetable
tt_target_time: ; Target clock time
.dc.w 0x0
tt_cur_repeat: ; repeat counter
.dc.b 0x0,0x0
tt_line: ; current scanline
.dc.b 0
tt_frame: ; current frame
.dc.b 0
tt_term_ptr: ; current position in termBuffer
.dc.w 0x0

#define tt_arg1 r23
#define tt_arg2 r24

.macro push_regs_r21_to_Z
  push YH
  push YL
  push ZH
	push ZL 
	;save flags & status register
	in ZL,_SFR_IO_ADDR(SREG)
  push ZL
  push XL
  push XH
  push r21
  push r22
  push r23
  push r24
  push r1
  push r2
.endm

.macro pop_regs_r21_to_Z
  pop r2
  pop r1
  pop r21
  pop r22
  pop r23
  pop r24
  pop XH
  pop XL
	pop ZL
	out _SFR_IO_ADDR(SREG),ZL
  pop ZL
  pop ZH
  pop YL
  pop YH
.endm

.macro push_regs_r14_to_Z
  push r14
  push r15
  push r16
  push r17
  push r18
  push r19
  push r20
  push_regs_r21_to_Z
.endm

.macro pop_regs_r14_to_Z
  pop_regs_r21_to_Z
  pop r20
  pop r19
  pop r18
  pop r17
  pop r16
  pop r15
  pop r14
.endm

.section .text

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; busy-wait 16-bit delay by X cycles inc call.  
;;; X can be any value 0 - 2**16
;;; Resolution is 4 cycles
;;; DESTROYS X
;;; Actual Scope Measurements F_CPU=28,636,360
;;; nop, 100ns (inc sbi?),  32=1.1us
;;; 100=3.5us 400=14us 1600=55us 6400=225us 25600=900us 
.macro delay
91:     ; loop is 4 per iteration - 1 at exit
    sbiw XL,4   ;2
    brcc 91b ;1/2
.endm

;;;;;;;;;;;;;;;;;;;
;; RULES FOR TIMETABLE ROW FUNCTIONS
;; X = current row delay
;; Y = tt_cur_ptr
;; Z = scratch
;; r0,r1,r22,r23,r24,r25 = scratch, use as needed
;; call tt_load_args IF NEEDED => r23,r24
run_timetable_row:
#ifdef DOMORSE
; DEBUG: Get Row Number
lds YL, tt_cur_ptr+0
lds YH, tt_cur_ptr+1
ldi XL, lo8(timetable)
ldi XH, hi8(timetable)
sub YH, XH
sbc YL, XL
lsr YL
lsr YL
repeatTapCharCI 50, 'R', YL
#endif

  ; Get Row 
  lds YL, tt_cur_ptr+0
  lds YH, tt_cur_ptr+1

  ; Store Row Delay
  ldd XL, Y+0
  ldd XH, Y+1

  ; Get Row Func
  ldd ZL, Y+2
  ldd ZH, Y+3

  ; Call Row Func
  icall  ; (Z)

  ; Increment and Store Row Pointer (may have changed)
  adiw YL, 6
  sts tt_cur_ptr+0, YL
  sts tt_cur_ptr+1, YH
  
  ; Return Delay in X
  ; Cleanup
  ret

.section .data
debug_cnt:
.word 0

.section .text

run_timetable:
  push_regs_r21_to_Z
  sbi _SFR_IO_ADDR(TRIG_DDR),TRIG_BIT
  sbi _SFR_IO_ADDR(DBG0_DDR),DBG0_BIT
  sbi _SFR_IO_ADDR(VID0_DDR),VID0_BIT
  sbi _SFR_IO_ADDR(VID1_DDR),VID1_BIT
;;setMorseSpeedScale 1
rtt_loop:
  call run_timetable_row
  ;; delay returned in X
repeatTapCharCI 70, 'D', XH
  ;cpi XL,0
  ;cpc XH,0
  ;breq rtt_loop ; delay == 0 - unnecessary?
  ;sbi _SFR_IO_ADDR(DBG0_PIN),DBG0_BIT
  delay 
  ;inc debug_cnt
  
  ;sbi _SFR_IO_ADDR(DBG0_PIN),DBG0_BIT
  jmp rtt_loop
  pop_regs_r21_to_Z
  ret

;;; TEST CASE
;;TIMER1_COMPA_vect:
  push_regs_r21_to_Z ;;; optimize me
  sbi _SFR_IO_ADDR(TRIG_DDR),TRIG_BIT
  sbi _SFR_IO_ADDR(DBG0_DDR),DBG0_BIT
  sbi _SFR_IO_ADDR(VID0_DDR),VID0_BIT
  sbi _SFR_IO_ADDR(VID1_DDR),VID1_BIT
 
  sbi _SFR_IO_ADDR(DBG0_PIN), DBG0_BIT
 
  lds XL, _SFR_MEM_ADDR(OCR1AL)
  lds XH, _SFR_MEM_ADDR(OCR1AH)

  ;ldi r21,0x10
  ;add XH,r21
  ;andi XH,0xf0
  ldi r21,0x01
  add XH,r21
  andi XH,0x0f
  brne 1f
    sbi _SFR_IO_ADDR(TRIG_PIN), TRIG_BIT;
    ldi XH, 0x01 ; don't want 0 == 65k
1:

  ;;; Set timer with new delay
  sts _SFR_MEM_ADDR(OCR1AH), XH
  sts _SFR_MEM_ADDR(OCR1AL), XL
 
  pop_regs_r21_to_Z
  reti  

;;; Test Interrupt time
;;TIMER1_COMPA_vect:
  call debug_trigger_toggle
  reti

TIMER1_COMPA_vect:
  push_regs_r21_to_Z ;;; optimize me
  sbi _SFR_IO_ADDR(TRIG_DDR),TRIG_BIT
  sbi _SFR_IO_ADDR(DBG0_DDR),DBG0_BIT
  sbi _SFR_IO_ADDR(VID0_DDR),VID0_BIT
  sbi _SFR_IO_ADDR(VID1_DDR),VID1_BIT

run_loop: 
  call run_timetable_row

  ; Don't return on small intervals, just loop
  ldi r22, 0x0
  ldi r23, 0xff
  cp  XL, r23
  cpc XH, r22
  brsh 9f
    lds r23, _SFR_MEM_ADDR(TCNT1L)
    lds r22, _SFR_MEM_ADDR(TCNT1H)
    sub XL, r23
    sbc XH, r22
    brcs 8f ; We're late!  No delay!
      ;sbi _SFR_IO_ADDR(DBG0_PIN),DBG0_BIT
      delay 
      ;sbi _SFR_IO_ADDR(DBG0_PIN),DBG0_BIT
8:
    ;reset timer as if interrupt happened
    clr r21
    sts _SFR_MEM_ADDR(TCNT1H), r21
    sts _SFR_MEM_ADDR(TCNT1L), r21
    jmp run_loop
9:

  ;ldi  r21, (1 << CS10) | (1 << WGM12)
  ;sts _SFR_MEM_ADDR(TCCR1B), r21

  ; We subtract a hack for INT overhead compensation
  ldi r22, 0
  ldi r23, 35 
  sub XL,r23
  sbc XH,r22
  sts _SFR_MEM_ADDR(OCR1AH), XH
  sts _SFR_MEM_ADDR(OCR1AL), XL
 
  pop_regs_r21_to_Z
  reti

sync_low:
cbi _SFR_IO_ADDR(DBG0_PORT),DBG0_BIT
  cbi _SFR_IO_ADDR(VID0_PORT),VID0_BIT
  cbi _SFR_IO_ADDR(VID1_PORT),VID1_BIT
  ret

sync_high:
sbi _SFR_IO_ADDR(DBG0_PORT),DBG0_BIT
  sbi _SFR_IO_ADDR(VID0_PORT),VID0_BIT
  cbi _SFR_IO_ADDR(VID1_PORT),VID1_BIT
  ret

sync_toggle:
sbi _SFR_IO_ADDR(DBG0_PIN),DBG0_BIT
  sbi _SFR_IO_ADDR(VID0_PIN), VID0_BIT
  cbi _SFR_IO_ADDR(VID1_PORT),VID1_BIT
  ret

next_frame:
  ldi r22,1
  sts tt_frame, r22
  ldi r22, 0
  sts tt_line, r22
  ldi r22, lo8(termBuffer)
  sts tt_term_ptr+0, r22
  ldi r22, hi8(termBuffer)
  sts tt_term_ptr+1, r22
  ret

debug_trigger_high:
  sbi _SFR_IO_ADDR(TRIG_DDR),TRIG_BIT
  sbi _SFR_IO_ADDR(TRIG_PORT),TRIG_BIT ; just for 'scope debug 
  ret

debug_trigger_low:
  sbi _SFR_IO_ADDR(TRIG_DDR),TRIG_BIT
  cbi _SFR_IO_ADDR(TRIG_PORT),TRIG_BIT
  ret

debug_trigger_toggle:
  sbi _SFR_IO_ADDR(TRIG_DDR),TRIG_BIT
  sbi _SFR_IO_ADDR(TRIG_PIN),TRIG_BIT
  ret

counted_loop:
  lds r22, tt_cur_repeat
  ldd r23, Y+4  ; backwards skip
  ldd r24, Y+5  ; repeat count

  ; Do we need to start our loop?
  cpi r22, 0
  brne not_eq_zero_cl
    mov r22, r24 ; load the repeat count
  not_eq_zero_cl:

  ; Did our loop just finish?
  ; Note, loop repeat of 1 is done without looping!
  cpi r22, 1
  brne not_eq_one_cl
    dec r22 ; last pass of loop now finished
    sts tt_cur_repeat, r22
    rjmp cleanup_cl
  not_eq_one_cl:

  ;; loop already started or just started
  dec r22  ; dec Repeat Count
  sts tt_cur_repeat, r22  ; Store repeat count

  ; Relocate Row Pointer
  inc r23 ; account for table increment
  ldi r25,6
  mul r23,r25
  sub YL, r0 ; move back row pointer
  sbc YH, r1 ; move back row pointer
cleanup_cl:
  ret

render_line:
  lds r22, tt_line
  lds r23, tt_frame
  ;call line_test_pattern
  ;call bw_test_pattern
  call term_line_render
  inc r22
  inc r23
  sts tt_line, r22
  sts tt_frame, r23
  ret

restart_timetable:
  sts tt_line, 0
  sts tt_frame, 0
#if 0
  ; Setup Terminal Character Buffer pointers
  ldi r22, lo8(termBuffer)
  sts tt_term_ptr+0, r22
  ldi r22, hi8(termBuffer)
  sts tt_term_ptr+1, r22
#endif
  ; Reset timetable pointer to start
  ldi YL, lo8(timetable)
  ldi YH, hi8(timetable)
  sbiw YL, 6 ; calling function will inc, so we must dec
#if 0
  ; DEBUG: put something in current_line0
  push ZH
  push ZL 
  ldi ZH, hi8(current_line0)
  ldi ZL, lo8(current_line0)
  ldi r24, 28
1:
    ldi r23, 0x0b
    st Z+, r23
    dec r24
    brne 1b
  pop ZL
  pop ZH
#endif
  ret

line_test_pattern:
  push r16
  push r17

  ldi r16,225 ;232
  ;sub r16,r22 ; r22 passed by render_line
bg_line_loop:
  ;;;out _SFR_IO_ADDR(DATA_PORT),r16
  ;out _SFR_IO_ADDR(DATA_PORT),r22
  dec r16
  brne bg_line_loop
  
  pop r17
  pop r16
  ret

bw_test_pattern:
  push r16
  push r17

  sbi _SFR_IO_ADDR(VID0_PORT),VID0_BIT
  ldi r16,30
  andi r22, 0x1
  brne 1f
  sbi _SFR_IO_ADDR(VID1_PORT),VID1_BIT
  nop
  nop
  nop
  nop
  nop
  nop
  nop
1:
bw_line_loop:
  cbi _SFR_IO_ADDR(VID1_PORT),VID1_BIT
  nop
  nop
  nop
  nop
  nop
  nop
  sbi _SFR_IO_ADDR(VID1_PORT),VID1_BIT
  nop
  nop
  nop
  dec r16
  brne bw_line_loop

  ldi r16,10
bw_line_loop2:
  cbi _SFR_IO_ADDR(VID1_PORT),VID1_BIT
  nop
  nop
  nop
  nop
  sbi _SFR_IO_ADDR(VID1_PORT),VID1_BIT
  dec r16
  brne bw_line_loop2
  
  pop r17
  pop r16
  ret

.macro jitter_fix9 tdiff
  ; Use the fact that brlo uses either 1 or 2 
  ; cycles based on comparison to un-jitter timing
	cpi \tdiff,1
	brlo .		;advance PC to next instruction
	cpi \tdiff,2
	brlo .		;advance PC to next instruction
	cpi \tdiff,3
	brlo .		;advance PC to next instruction
 	cpi \tdiff,4
	brlo .		;advance PC to next instruction
	cpi \tdiff,5
	brlo .		;advance PC to next instruction
	cpi \tdiff,6
	brlo .		;advance PC to next instruction
	cpi \tdiff,7
	brlo .		;advance PC to next instruction
	cpi \tdiff,8
	brlo .		;advance PC to next instruction
	cpi \tdiff,9
	brlo .		;advance PC to next instruction
.endm

test_delay:
  call syncPulseA1

  sbi _SFR_IO_ADDR(DBG0_DDR),DBG0_BIT
  sbi _SFR_IO_ADDR(TRIG_DDR),TRIG_BIT

  sbi _SFR_IO_ADDR(DBG0_PORT),DBG0_BIT
  nop
  sbi _SFR_IO_ADDR(DBG0_PIN),DBG0_BIT

  ldi XH, 0
  ldi XL, 0x20
  delay

  sbi _SFR_IO_ADDR(DBG0_PIN),DBG0_BIT
  ldi XH, 0
  ldi XL, 0x64
  delay
  sbi _SFR_IO_ADDR(DBG0_PIN),DBG0_BIT
  
  lsl XL
  rol XH
  lsl XL
  rol XH
  delay
  sbi _SFR_IO_ADDR(DBG0_PIN),DBG0_BIT

  lsl XL
  rol XH
  lsl XL
  rol XH
  delay
  sbi _SFR_IO_ADDR(DBG0_PIN),DBG0_BIT

  lsl XL
  rol XH
  lsl XL
  rol XH
  delay
  cbi _SFR_IO_ADDR(DBG0_PORT),DBG0_BIT

  lsl XL
  rol XH
  lsl XL
  rol XH
  delay
  cbi _SFR_IO_ADDR(DBG0_PORT),DBG0_BIT

  ; Should be near overflow limit now

  ret

;;;; Render a TerminalPlatform line 
term_line_render:
  push_regs_r14_to_Z

  #define r22_tt_line r22

  #define ZH_current_line0 ZH
  #define ZL_current_line0 ZL
  ldi ZH_current_line0, hi8(current_line0)
  ldi ZL_current_line0, lo8(current_line0)

  ldi XH, hi8(tt_term_ptr)
  ldi XL, lo8(tt_term_ptr)
  #define YL_cur_char YL
  #define YH_cur_char YH
  #define Y_cur_char Y
  ld YL_cur_char, X+   ; Y = pointer to current terminal line - FIXME
  ld YH_cur_char, X

  #define X_font_bitpacked X
  #define XH_font_bitpacked XH
  #define XL_font_bitpacked XL
  ldi XH_font_bitpacked, hi8(font_bitpacked)
  ldi XL_font_bitpacked, lo8(font_bitpacked)

  ; determine what pixel line of this character row we are at
  #define r17_line_mod r17
  mov r17_line_mod, r22_tt_line
  ; FIXME ASSUMPTION: FONT_TILESET_HEIGHT == 16
  and r17_line_mod, 0xf  ; tt_line % 16 ( = FONT_TILESET_HEIGHT )

  ; offset font pointer by pixel line - assumes 8-bit width
  ldi r16, 0
  add XL_font_bitpacked, r17_line_mod
  adc XH_font_bitpacked, r16

  #define r16_FONT_TILE_SIZE r16
  ; FIXME ASSUMPTION: assumes font width=8 bits
  ldi r16_FONT_TILE_SIZE, FONT_TILESET_HEIGHT 

  #define r18_TERM_WIDTH r18 
  ldi r18_TERM_WIDTH, 1 ;;TERM_WIDTH ;; FIXME DEBUG - set wrong for debug
  #define r15_CHAR_COUNTDOWN r15
  mov r15_CHAR_COUNTDOWN, r18_TERM_WIDTH
  ;;;ldi r18,10 ;232
101:
  #define r25_char r25
  ld r25_char, Y_cur_char+ ; copy character code from terminal buffer
  push XL_font_bitpacked    ; lookup character code line in font
  push XH_font_bitpacked
  mul r25_char, r16_FONT_TILE_SIZE  ; calc pointer to bitmap
  add XL_font_bitpacked, r0
  adc XH_font_bitpacked, r1
  ld r25, X_font_bitpacked ; load 8-bits of font raster
  pop XH ; restore font_bitpacked pointers
  pop XL
  
  #define r23_bits r23
  ;ld r23,Z+  ;; DEBUG: this is how it should work
  mov r23_bits, r25  ; DEBUG: Loading direct for now, no "next" line
  ldi r24, 8
  ldi r26, 0xff
  byte_loop: 
    mov r25, r23_bits
    andi r25, 0x1
    brne 11f ; if 0
      sbi _SFR_IO_ADDR(VID0_PORT), VID0_BIT
      cbi _SFR_IO_ADDR(VID1_PORT), VID1_BIT
    rjmp 12f ; else 1
11:
      sbi _SFR_IO_ADDR(VID0_PORT), VID0_BIT
      sbi _SFR_IO_ADDR(VID1_PORT), VID1_BIT
12:
    lsr r23_bits
    dec r24
    brne byte_loop
  dec r15_CHAR_COUNTDOWN
  brne 101b

  ; set back to black
  sbi _SFR_IO_ADDR(VID0_PORT), VID0_BIT
  cbi _SFR_IO_ADDR(VID1_PORT), VID1_BIT

cleanup_tlr:
  pop_regs_r14_to_Z
  ret


