#include "WikiApp.h"
#include "vt52.h"
#include <string.h>
#include "crc32.h"
#include "bgUtil.h"
#include "bgUtil2.h"

typedef struct {
  Cursor begin; // cursor pointing to the beginning of this sequence
  Cursor c; // current cursor position
  int level;
  WikiApp *wa;
} WikiUserData;

void BeginSectionList(Level *lev) {
  WikiUserData *wud = (WikiUserData *) lev->data;
  wud->c = CursorDup(&wud->begin);
}

void DestroySectionList(Level *lev) {

}

int RenderArticleList(Level *lev, int index, FILE *outFile) {
  return -1;
}

int RenderSectionList(Level *lev, int index, FILE *outFile) {
  //fprintf(stderr, "RenderSectionList(%i, %x)\n",index,  outFile);
  WikiUserData *wud = (WikiUserData *) lev->data;
  uint32_t found;
  uint32_t level;
  do {
    found = CursorNext(&wud->c);
    //fprintf(stderr, "%i = CursorNext(lev=%i, idx=%i)\n",found, wud->level, index);
    if (found == BAInvalid)
      return -1;
    FileOpenRO(wud->wa->pageContext.blobFname);
    ERRORassert();
    FileSeek(found);
    level = UnpackIntFromFile(FileGetHandle());
    if (level < wud->level) // return to parent level
      return -1;
  } while (level > wud->level);
  // Read ixss blob record
  uint32_t parent = UnpackUIntFromFile(FileGetHandle());
  //printf(" level = %i, parent = %08x\n ", (int) level, (unsigned int) parent);
  int sz=level + 1;
  if (!outFile) {
    sz += UnpackUIntFromFile(FileGetHandle());
    ERRORassert();
    return sz;
  }
  for (int i=0; i<level; ++i) {
    fprintf(outFile, "+");
  }
  fprintf(outFile, " ");
  ERRORassert();
  sz += UnpackAndPrintStringFromFile(FileGetHandle(), outFile);
  return sz;
}

int RenderArticleText(Level *lev, int index, FILE * outFile) {
  return -1;
}

Level ExpandSectionList(Level *lev, int index) {
  Level sublev = LevelCreate(RenderSectionList, ExpandSectionList, BeginSectionList, 0x0);
  WikiUserData *wud = (WikiUserData *) lev->data;
  WikiUserData *subwud = (WikiUserData *) sublev.data;
  LevelSetPosition(lev, index+1);
  subwud->begin = CursorDup(&wud->c);
  subwud->wa = wud->wa;
  subwud->level = wud->level + 1;
  return sublev;
}

Level ExpandArticleList(Level *lev, int index) {
  Level sublev = LevelCreate(RenderSectionList, ExpandSectionList, BeginSectionList, 0x0);

  return sublev;
}


void WikiAppInit(WikiApp *wa) {
  assert(sizeof(WikiUserData) <= LEVEL_DATA_SIZE);
  wa->pageContext.tableFname = "zb24.bgt";
  wa->pageContext.blobFname  = "zb24.bgb";
  wa->pageContext.fileStateVersion = -1;
  TreeInit(&wa->pageTree, &wa->pageContext);
  HierInit(&wa->hier, 0x0);
  wa->searchPos = 0;
}

void WikiAppTitleSearch(WikiApp *wa, const char *title) {
  /* Hash title string */
  KeyT key = CalcCRC(title, strlen(title));
  /* Perform search for title hash */
  printf("\nSearching...\n");
  Cursor c = SelectConsecutiveKeys(wa->pageTree, key);
  printf("Search Complete\n");
  Level lev = LevelCreate(RenderSectionList, ExpandSectionList, BeginSectionList, 0x0); 
  WikiUserData *wud = (WikiUserData *) lev.data;
  wud->begin = c;
  wud->wa = wa;
  wud->level = 1;
  wa->hier.levels[0] = lev;
  wa->hier.depth = 1;
}
