/** WikiText processing and filtering routines **/
#ifndef WIKITEXT_H_
#define WIKITEXT_H_ 1

#include <stdint.h>

/** These tokens are compared in order, so if one token is a prefix of another, place it later */
enum WTTypes { // 2-bit type code for tokens, specifying if token opens or closes a sub-context, is straight text, or is flat
  WT_FLAT = 0, // token has no effect on nested context
  WT_OPEN = 1, // token opens new nested context
  WT_CLOSE = 2,// token closes new nested context
  WT_MATCH = 3 // token opens OR closes nested context, matches prior/later tag
};

enum WTIds {
  WT_ITALIC,
  WT_BOLD,
  WT_BOLD_ITALIC,
  WT_HEAD1,
  WT_HEAD2,
  WT_HEAD3,
  WT_HEAD4,
  WT_HEAD5,
  WT_HEAD6,
  WT_HR,
  WT_BULLET_LIST,
  WT_NUMBER_LIST,
  WT_DEF_LIST,
  WT_HTML_TAG,
  WT_PRE,
  WT_INT_LINK,
  WT_EXT_LINK,
  WT_TEMPLATE,
  WT_NEWLINE,
  WT_PIPE,
  WT_EQUALS,
  WT_TEXT
};

typedef struct {
  char *str;
  unsigned int strLen:8;
  unsigned int type:2; 
  unsigned int id:6;
} WToken;

typedef WToken WSegment;

WToken *WTFirstChild(WToken *tokens, int tokensLen);
WToken *WTGetChild(WToken *tokens, int tokensLen, int n);
WToken *WTNextSibling(WToken *tokens, int tokensLen);
int WTTokenize(char *text, int textLen, WToken *tokens, int tokensLen);

/** Return the next "flat" token.
 *  Example tokens: '[[', ']]', '\n', '{{', '}}'. '|', '', ''', '<', '>'
 *  We do NOT assume that all tokens are meaninful, we just want to divide the text into
 *  all POSSIBLE tokens and can sort out the context later. **/
WToken WikiTextNextToken(char *text, int textLen);
WSegment WikiTextSegment(char *text, int textLen, WSegment *segments, int segmentsSize);

int WTPrintDebug(WToken *tokens, int tokensLen, int level);
#endif // WIKITEXT_H_
