#ifndef BGCORE_H_
#define BGCORE_H_ 1

#include "bgDefines.h"

typedef struct {
  BlockAddrT topNode;
} BgTree;

/** References a row within the tree. **/
typedef struct {
  BgTree tree;
  BAT node;
  int index;
} RowPtr;

void InitNode(Node *n);

/* Inefficient, should probably stash parent index in Node
 * structure. */
uint16_t FindIndexOfChild(BAT parent, BAT child);

typedef int (*NodeVisitor)(BAT block, Node *node);
BAT NodeVisitDFS(NodeVisitor func, BAT node);
/** Returns number of valid internal elements in a node **/
int GetNodeLen(BAT nodeAddr);
uint8_t NodeDepth(BAT node);
int NodeIsLeaf(BAT node);
BAT NodeItem(BAT node, int i);
BAT NodeKey(BAT node, int i);
BAT NodeParent(BAT node);
/** Finds the left-most leaf child node (assumes that there IS one) **/
BAT NodeFindFirstChild(BAT node);

int SetKeysVisitor(BAT block, Node *n);
BAT FindInternal(BAT nodeAddr, KeyT key, BAT *outParent, int *outIndex);

/** Calculates the number of bytes to hold a tree for a set number of elements
 *  May be slightly larger than actually required (by a few blocks) **/
int CalcTreeSize2(int numItems, int depth);

/** Calculate the Minimum tree depth given the number of items to store and the
 * tree parameters.  Might be a little conservative (aka, sloppy). **/
int CalcMinimumDepth(int numItems);

BAT NodeNextLeaf(BAT node, BAT child);

RowPtr RowPtrInit(BgTree table);
RowPtr RowPtrNext(RowPtr row);

/** Returns the last leaf node under the spcified node.  depth = 0 = leaf node **/
BAT GetLastNodeOfDepth(BAT node, uint8_t depth);

/** Adds a new node to a full tree **/
BAT AppendNewLeafNode(BAT topNode);

#endif /* BGCORE_H_ */
