#ifndef GLYPHS_HPP_
#define GLYPHS_HPP_ 1
#ifdef __cplusplus
 extern "C" {
#endif 

#include <stdint.h>
#include <string.h>
#include "Defines.hpp"

#define PIXEL_WIDTH 240
#define PIXEL_HEIGHT 224
/** How wide is the "natural" size of each cell (doesn't need to be 8 **/
#define CELL_X_ADVANCE 6
#define CELL_Y_ADVANCE 8
#define CELL_X_MAX_SHIFT 15
#define CELL_Y_MAX_SHIFT 15
#define CELLMAP_WIDTH  (PIXEL_WIDTH/CELL_X_ADVANCE)
#define CELLMAP_HEIGHT (PIXEL_HEIGHT/CELL_Y_ADVANCE)

extern const uint8_t *glyphMap; // bit packed glyph array
extern uint8_t glyphMapSize; // number of GLYPHS in array (NOT bytes)
extern uint8_t glyphWidth;
extern uint8_t glyphHeight;

#define mod(x, y) ((y + (x % y)) % y)

/** Some of this math is tricky, so we centralize it **/
#define RenderBufferWidth() (PIXEL_WIDTH)

#define RenderBufferHeight() (3*CELL_Y_ADVANCE)

#define RenderBufferSize() (PIXEL_WIDTH * 3 * CELL_Y_ADVANCE)

/** Because RenderBuffer is circular, this is a little tricky.  I will probably
 * also change this to a bit-packed type later **/
void RenderBufferSet(unsigned char *buf, int x, int y, uint8_t value);

uint8_t RenderBufferGet(unsigned char *buf, int x, int y);

/** This is a circular buffer **/
unsigned char *CalcRenderBufferLinePos(unsigned char *buf, int y);

/** 2 bytes per Cell, specify glyph offset, index, mask invert, and brightness **/
typedef struct {
  unsigned int shiftX : 3; //< Right shift / 2
  unsigned int shiftY : 3; //< Downward shift / 2
  unsigned int bright : 1; //< We assume two tone greyscale.  1 for white, 0 for grey.
  unsigned int invert : 1; //< invert the glyph and glyph-mask
  uint8_t glyphIndex;
} Cell;

void CellInit(Cell *c);

extern Cell cellMap[CELLMAP_WIDTH][CELLMAP_HEIGHT];
/** The cell map offsets can be used to scroll.  The buffer is toroidal. **/
extern int cellMapXOffset;
extern int cellMapYOffset;

Cell *CellMapGet(int x, int y);

void CellMapSet(int x, int y, Cell c);

#define GetGlyph(idx) ( glyphMap + idx*(glyphWidth*glyphHeight/8) )

void RenderRow(unsigned char *b, int row);

/** Add a Glyph to the given x/y pixel location **/
void AddGlyph(int x, int y, uint8_t glyphIndex, uint8_t bright, uint8_t invert);

void DebugPrintGlyphs();

void ClearCells();

void SetGlyphMap(const uint8_t *glyphMap_, uint8_t glyphMapSize_, uint8_t glyphWidth_, uint8_t glyphHeight_);

void RenderCell8bpp(Cell c, unsigned char *b, int x, int y);

#ifdef __cplusplus
}
#endif 
#endif // GLYPHS_HPP_
