
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>  
#include <avr/sleep.h>

#include "z810-defines.h"

void tapString(const char *s);

void sounder(int onOrOff, int msec) {
  DDRD  |= 1 << PD4;
  if (onOrOff)
    PORTD &= ~(1 << PD4); // led on
  else 
    PORTD |= 1 << PD4; // led off
  _delay_ms((float)msec);
  PORTD |= 1 << PD4; // off at end
}

  extern unsigned char burstOffset;
	extern unsigned char curr_frame;
	extern unsigned char vsync_phase;

void init() {
	//stop timers
	TCCR1B=0;
	TCCR0B=0;

	//set ports
	DDRC=0xff; //video dac
	DDRB=0xff; //h-sync for ad725

	//set sync generator counter on TIMER1
	OCR1AH=HDRIVE_CL_TWICE>>8;
	OCR1AL=HDRIVE_CL_TWICE&0xff;

	TCCR1B=(1<<WGM12)+(1<<CS10);//CTC mode, use OCR1A for match
	TIMSK1=(1<<OCIE1A);			//generate interrupt on match

	//set clock divider counter for AD725 on TIMER0
	//outputs 14.31818Mhz (4FSC)
	TCCR0A=(1<<COM0A0)+(1<<WGM01); //toggle on compare match + CTC
	OCR0A=0; //divide main clock by 2
	TCCR0B=(1<<CS00); //enable timer, no pre-scaler

	SYNC_PORT=(1<<SYNC_PIN)|(1<<VIDEOCE_PIN); //set sync & chip enable line to hi

  burstOffset=0;
	curr_frame=0;
	vsync_phase=0;
	
  sei();

}

int main() {
  tapStringAsm("S ");
  init();
  for (int i=0; 1; ++i) {
    char c[2];
    c[1] = (i % 42) + '0';
    c[2] = 0x0;
    tapStringAsm(c);
    tapStringAsm("          ");
  }
}
