#include <avr/io.h>
#include "z810-defines.h"

.global TIMER1_COMPA_vect
.global SetColorBurstOffset
.global burstOffset
.global vsync_phase
.global curr_frame
.global sync_pulse
.global sync_phase
.global curr_field

.section .bss
burstOffset:		.byte 1
sync_phase:  .byte 1 ;0=pre-eq, 1=eq, 2=post-eq, 3=hsync, 4=vsync
sync_pulse:	 .byte 1
vsync_flag:  .byte 1	;set 30 hz
curr_field:	 .byte 1	;0 or 1, changes at 60hz
curr_frame:  .byte 1	;odd or even frame

vsync_phase:    .byte 1

.section .text

;;;; TIMER1_COMPA_vect IS NOT A LABEL, it is an address def in avr/io headers!
;***************************************************************************
; Video sync interrupt
; 4 cycles to invoke 
;
;***************************************************************************
TIMER1_COMPA_vect:
call toggleA3
	push ZH;2
	push ZL;2
	;save flags & status register
	in ZL,_SFR_IO_ADDR(SREG);1
	push ZL ;2		

	;Read timer offset since rollover to remove cycles 
	;and conpensate for interrupt latency.
	;This is nessesary to eliminate frame jitter.

	lds ZL,_SFR_MEM_ADDR(TCNT1L)
	subi ZL,0x0e ;MIN_INT_LATENCY

	cpi ZL,1
	brlo .		;advance PC to next instruction

	cpi ZL,2
	brlo .		;advance PC to next instruction

	cpi ZL,3
	brlo .		;advance PC to next instruction

 	cpi ZL,4
	brlo .		;advance PC to next instruction

	cpi ZL,5
	brlo .		;advance PC to next instruction

	cpi ZL,6
	brlo .		;advance PC to next instruction

	cpi ZL,7
	brlo .		;advance PC to next instruction
	
	cpi ZL,8
	brlo .		;advance PC to next instruction

	cpi ZL,9
	brlo .		;advance PC to next instruction

	rcall sync
	;call update_sound_buffer
  call delay63cyc

	pop ZL
	out _SFR_IO_ADDR(SREG),ZL
	pop ZL
	pop ZH
	reti

.section .text
	
	sync_func_vectors:	.word pm(do_pre_eq)
						.word pm(do_eq)
						.word pm(do_post_eq)
						.word pm(do_hsync)


;***************************************************
; Composite SYNC
;***************************************************

sync:
	push r0
	push r1
		
  ;; dispatch to proper vector based on sync phase	
	ldi ZL,lo8(sync_func_vectors)	
	ldi ZH,hi8(sync_func_vectors)

	lds r0,sync_phase
	lsl r0 ;*2
	clr r1
	
	add ZL,r0
	adc ZH,r1
	
	lpm r0,Z+
	lpm r1,Z
	movw ZL,r0

	icall	;call sync functions

	pop r1
	pop r0
	ret


;*************************************************
; Generate a H-Sync pulse - 136 clocks (4.749us)
; Note: TCNT1 should be equal to 
; 0x44 on the cbi 
; 0xcc on the sbi 
;
; Cycles: 144
; Destroys: ZL (r30)
;*************************************************
hsync_pulse:
	cbi _SFR_IO_ADDR(SYNC_PORT),SYNC_PIN ;2
	
	;;;call update_sound_buffer ;36 -> 63
  call delay63cyc
	
	ldi ZL,21
	dec ZL 
	brne .-4



	lds ZL,sync_pulse
	dec ZL
	sts sync_pulse,ZL

	sbi _SFR_IO_ADDR(SYNC_PORT),SYNC_PIN ;2

	nop
	nop

	ret

;;; delay 63 cycles (including call) - bcg'09
delay63cyc:
  ;rcall ate 4 =59
  push r16   ;2 =57
  ldi r16,16 ;1 =56
loop63: ;48 left to burn here, 3 cycles per loop
  dec r16    ;1 
  brne loop63;1/2
  pop r16    ;2
  ret       ;5

;**************************
; PRE_EQ pulse
; Note: TCNT1 should be equal to 
; 0x44 on the cbi
; 0x88 on the sbi
; pulse duration: 68 clocks
;**************************
do_pre_eq:
	cbi _SFR_IO_ADDR(SYNC_PORT),SYNC_PIN ; HDRIVE sync pulse low

	;;;call update_sound_buffer_2 ;36 -> 63
  call delay63cyc

	sbi _SFR_IO_ADDR(SYNC_PORT),SYNC_PIN ;68
	nop

	ldi ZL,SYNC_PHASE_EQ
	ldi ZH,SYNC_EQ_PULSES
	rcall update_sync_phase

	rcall set_double_rate_HDRIVE

	ret

;************
; Serration EQ
; Note: TCNT1 should be equal to 
; 0x44  on the cbi
; 0x34A on the sbi
; low pulse duration: 774 clocks
;************
do_eq:
	cbi _SFR_IO_ADDR(SYNC_PORT),SYNC_PIN ; HDRIVE sync pulse low

	;;;;call update_sound_buffer_2 ;36 -> 63
  call delay63cyc

	ldi ZL,181-9+4
do_eq_delay:
	nop
	dec ZL
	brne do_eq_delay ;135

	nop
	nop

	sbi _SFR_IO_ADDR(SYNC_PORT),SYNC_PIN ;136

	ldi ZH,SYNC_POST_EQ_PULSES
	ldi ZL,SYNC_PHASE_POST_EQ
	rcall update_sync_phase
		
	ret

;************
; POST_EQ
; Note: TCNT1 should be equal to 
; 0x44 on the cbi
; 0x88 on the sbi
; pulse cycles: 68 clocks
;************
do_post_eq:
	cbi _SFR_IO_ADDR(SYNC_PORT),SYNC_PIN ; HDRIVE sync pulse low

	;;;;call update_sound_buffer_2 ;36 -> 63
  call delay63cyc

	sbi _SFR_IO_ADDR(SYNC_PORT),SYNC_PIN ;68

	nop

	ldi ZL,SYNC_PHASE_HSYNC
	ldi ZH,SYNC_HSYNC_PULSES
	rcall update_sync_phase



	lds ZL,sync_pulse
	cpi ZL,(SYNC_POST_EQ_PULSES-1)
	brne noshift
	//cause a shift in the color burst phase
	//on odd frames (NTSC superframe?)
	lds ZL,curr_field
	cpi ZL,1
	nop
	
	lds ZH,burstOffset
	brne peq_odd
	lds ZH,burstOffset
	neg ZH
 peq_odd:

	ldi ZL,hi8(HDRIVE_CL_TWICE) //4
	sts _SFR_MEM_ADDR(OCR1AH),ZL	
	
	ldi ZL,lo8(HDRIVE_CL_TWICE) //4
	add ZL,ZH
	sts _SFR_MEM_ADDR(OCR1AL),ZL
	ret

noshift:
	;restore full line size
	ldi ZL,hi8(HDRIVE_CL_TWICE)
	sts _SFR_MEM_ADDR(OCR1AH),ZL	
	ldi ZL,lo8(HDRIVE_CL_TWICE)
	sts _SFR_MEM_ADDR(OCR1AL),ZL

	ret

;************
; update sync phase
; ZL=next phase #
; ZH=next phases's pulse count
;
; returns: ZL: 0=more pulses in phase
;              1=was the last pulse in phase
;***********
update_sync_phase:

	lds r0,sync_pulse
	dec r0
	lds r1,_SFR_MEM_ADDR(SREG)
	sbrc r1,SREG_Z
	mov r0,ZH
	sts sync_pulse,r0	;set pulses for next sync phase

	lds r0,sync_phase
	sbrc r1,SREG_Z
	mov r0,ZL			;go to next phase 
	sts sync_phase,r0
	
	ldi ZL,0
	sbrc r1,SREG_Z
	ldi ZL,1

	ret

;**************************************
; Set HDRIVE to double rate during VSYNC
;**************************************
set_double_rate_HDRIVE:

	ldi ZL,hi8(HDRIVE_CL_TWICE)
	sts _SFR_MEM_ADDR(OCR1AH),ZL
	
	ldi ZL,lo8(HDRIVE_CL_TWICE)
	sts _SFR_MEM_ADDR(OCR1AL),ZL

	ret

;**************************************
; Set HDRIVE to normal rate
;**************************************
set_normal_rate_HDRIVE:

	ldi ZL,hi8(HDRIVE_CL)
	sts _SFR_MEM_ADDR(OCR1AH),ZL
	
	ldi ZL,lo8(HDRIVE_CL)
	sts _SFR_MEM_ADDR(OCR1AL),ZL

	ret

;************
; HSYNC
;************
do_hsync:
	cbi _SFR_IO_ADDR(SYNC_PORT),SYNC_PIN ; HDRIVE sync pulse low

	;;;;call update_sound_buffer ;36 -> 63
  call delay63cyc

	ldi ZL,32-9
do_hsync_delay:
	dec ZL
	brne do_hsync_delay ;135

	sbi _SFR_IO_ADDR(SYNC_PORT),SYNC_PIN ;136

	rcall set_normal_rate_HDRIVE


	ldi ZL,SYNC_PHASE_PRE_EQ
	ldi ZH,SYNC_PRE_EQ_PULSES

	rcall update_sync_phase

	sbrs ZL,0
	rcall render

	sbrs ZL,0
	rjmp not_start_of_frame


	;call sound mixing if first vsync pulse
	;;;;call MixSound

not_start_of_frame:

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Scratch video play - bcg'09

render_bg_line:
  push r16
  push r17

  ldi r16,222
bg_line_loop:
  nop
  nop
  nop
  out _SFR_IO_ADDR(DATA_PORT),r16
  dec r16
  brne bg_line_loop
  
  pop r17
  pop r16
  ret


;;; Scratch video play
sub_video_mode_bg:

	;waste line to align with next hsync in render function
	ldi ZL,222 
modebg_render_delay:
	lpm
	nop
	dec ZL
	brne modebg_render_delay 

	lpm
	lpm





  ldi r16,224 ; scan lines to output
  mov r10,r16







next_line_bg:
  rcall hsync_pulse ; 3+144=147

	ldi r19,37
	dec r19			
	brne .-4

	;***draw line***
	call render_bg_line

	ldi r19,26
	dec r19			
	brne .-4


  dec r10
  breq bg_frame_end ; last line?










  rjmp next_line_bg













bg_frame_end:

	ldi r19,5
	dec r19			
	brne .-4
	rjmp .

  rcall hsync_pulse ;145

bg_text_end2:

	;set vsync flag if beginning of next frame (each two fields)
	ldi r17,1
	lds r16,curr_field
	eor r16,r17
	sts curr_field,r16
	sbrs r16,0
	sts vsync_flag,r17

	;clear any pending timer int
	ldi ZL,(1<<OCF1A)
	sts _SFR_MEM_ADDR(TIFR1),ZL

  ret

render:
	push ZL

	lds ZL,sync_pulse
	cpi ZL,SYNC_HSYNC_PULSES - FIRST_RENDER_LINE
	brsh render_end
	cpi ZL,SYNC_HSYNC_PULSES - FIRST_RENDER_LINE - 224
	brlo render_end
	
	push r2
	push r3
	push r4
	push r5

	push r6
	push r7
	push r8
	push r9

	push r10
	push r11
	push r12
	push r13

	push r14
	push r15
	push r16
	push r17

	push r18
	push r19
	push r20
	push r21

	push r22
	push r23
	push r24
	push r25

	push XL
	push XH
	push YL
	push YH 
		
	
  call sub_video_mode_bg

	pop YH
	pop YL
	pop XH
	pop XL

	pop r25
	pop r24
	pop r23
	pop r22

	pop r21
	pop r20
	pop r19
	pop r18

	pop r17
	pop r16
	pop r15
	pop r14

	pop r13
	pop r12
	pop r11
	pop r10

	pop r9
	pop r8
	pop r7
	pop r6

	pop r5
	pop r4
	pop r3
	pop r2

render_end:
	pop ZL
	ret

renderend:

