
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>  
#include <avr/sleep.h>

#include "z812-defines.h"

void tapString(const char *s);

void sounder(int onOrOff, int msec) {
  DDRD  |= 1 << PD4;
  if (onOrOff)
    PORTD &= ~(1 << PD4); // led on
  else 
    PORTD |= 1 << PD4; // led off
  _delay_ms((float)msec);
  PORTD |= 1 << PD4; // off at end
}

  extern unsigned char burstOffset;
	extern unsigned char curr_frame;
	extern unsigned char vsync_phase;

void init() {
	//stop timers
	TCCR1B=0;
	TCCR0B=0;

	//set ports
	DDRC=0xff; //video dac
	DDRB=0xff; //h-sync for ad725

  //  16-BIT TIMER 1 HANDLING VIDEO INTERRUPTS
	//set sync generator counter on TIMER1
  // Sets OCR to 909 = 32 usec = 1/2 video line
	OCR1AH=HDRIVE_CL_TWICE>>8;
	OCR1AL=HDRIVE_CL_TWICE&0xff;

	TCCR1B=(1<<WGM12)+(1<<CS10);//CTC mode, use OCR1A for match
	TIMSK1=(1<<OCIE1A);			//generate interrupt on match

  //  8-BIT TIMER 0 CLOCKING THE NTSC CHIP
	//set clock divider counter for AD725 on TIMER0
	//outputs 14.31818Mhz (4FSC)
  //  COM = Compare Output Mode - enable output pin. Must also set DDR.
  //  WGM = Waveform Generation Mode - here set for CTC
  //  CTC = Clear Timer on Compare - restart at 0 when OCR value hit
	TCCR0A=(1<<COM0A0)+(1<<WGM01); //toggle on compare match + CTC
  // OCR value of 0 will match every other cycle, and CTC reset
	OCR0A=0; //divide main clock by 2
  // TCCR = Timer/Counter Control Reg - CS = Clock Select bits
	TCCR0B=(1<<CS00); //enable timer, no pre-scaler

	SYNC_PORT=(1<<SYNC_PIN)|(1<<VIDEOCE_PIN); //set sync & chip enable line to hi

  burstOffset=0;
	curr_frame=0;
	vsync_phase=0;
	
  ///////////////sei();

}

int main() {
  //tapString("S ");
  init();
  while (1)
    run_timetable();
  for (int i=0; 1; ++i) {
    char c[2];
    c[1] = (i % 42) + '0';
    c[2] = 0x0;
//    tapString(c);
//    tapString("          ");
  }
}
