#include <avr/io.h>
#include "z810-defines.h"

.global TIMER1_COMPA_vect
.global SetColorBurstOffset
.global burstOffset
.global vsync_phase
.global curr_frame
.global sync_pulse
.global sync_phase
.global curr_field
.global run_timetable

.section .bss
burstOffset:		.byte 1
sync_phase:  .byte 1 ;0=pre-eq, 1=eq, 2=post-eq, 3=hsync, 4=vsync
sync_pulse:	 .byte 1
vsync_flag:  .byte 1	;set 30 hz
curr_field:	 .byte 1	;0 or 1, changes at 60hz
curr_frame:  .byte 1	;odd or even frame

vsync_phase:    .byte 1

#ifndef F_CPU
#  define F_CPU 28636360
#endif

//define nsec2cyc(n) ((n*F_CPU)/1000000000)
#define nsec2cyc(n) (((n*F_CPU)/1000000)/1000)

.section .data

;;;; Transition table
;;;; flags, count, DELAY1, DELAY2
timetable:
; pre-eq phase
.dc.b 0x0, 6
.dc.w nsec2cyc(4700)                   ;pre-eq 1 low
.dc.w HDRIVE_CL_TWICE - nsec2cyc(4700) ;pre-eq 1 high
; vert sync/eq pulses
.dc.b 0x0, 6
.dc.w HDRIVE_CL_TWICE - nsec2cyc(4700) ;vsync low 1
.dc.w nsec2cyc(4700)                   ;vsync high
; post-eq pulses
.dc.b 0x0, 6
.dc.w nsec2cyc(4700)                   ;post-eq 1 low
.dc.w HDRIVE_CL_TWICE - nsec2cyc(4700) ;post-eq 1 high
; after this, normal hsync lines start
.dc.b 0x1, 255 
.dc.w nsec2cyc(4700) 
.dc.w HDRIVE_CL - nsec2cyc(4700)
.dc.b 0x1, 6 ; couldn't count to 262 in one byte
.dc.w nsec2cyc(4700) 
.dc.w HDRIVE_CL - nsec2cyc(4700)
.dc.b 0x1, 1 ; half line
.dc.w nsec2cyc(4700)
.dc.w HDRIVE_CL_TWICE - nsec2cyc(4700)
.dc.b 0x80, 0  ; 0x80 flag for end/restart
.dc.w 0 , 0

;;; State for interrupt-driven timetable func
tt_cur_ptr: ; Ptr to current timetable entry
.dc.w 0x0
tt_target_time: ; Target clock time
.dc.w 0x0
tt_cur_repeat: ; repeat counter
.dc.b 0x0
tt_cur_stage:  ; 0 for first timing, 1 for second
.dc.b 0x0  ; dummy alignment byte

.section .text

;;;; TIMER1_COMPA_vect IS NOT A LABEL, it is an address def in avr/io headers!

TIMER1_COMPA_vect:
	push ZH;2
	push ZL;2
	;save flags & status register
	in ZL,_SFR_IO_ADDR(SREG);1
	push ZL ;2		
  push r23

	;Read timer offset since rollover to remove cycles
	;and conpensate for interrupt latency.
	;This is nessesary to eliminate frame jitter.
	lds ZL,_SFR_MEM_ADDR(TCNT1L)
	subi ZL,0xe ;MIN_INT_LATENCY

  ; Use the fact that brlo uses either 1 or 2 
  ; cycles based on comparison to un-jitter timing
	cpi ZL,1
	brlo .		;advance PC to next instruction
	cpi ZL,2
	brlo .		;advance PC to next instruction
	cpi ZL,3
	brlo .		;advance PC to next instruction
 	cpi ZL,4
	brlo .		;advance PC to next instruction
	cpi ZL,5
	brlo .		;advance PC to next instruction
	cpi ZL,6
	brlo .		;advance PC to next instruction
	cpi ZL,7
	brlo .		;advance PC to next instruction
	cpi ZL,8
	brlo .		;advance PC to next instruction
	cpi ZL,9
	brlo .		;advance PC to next instruction


;  ldi r23,'A'
;  call tapCharAsm
;  adiw ZL,'0'
;  mov r23,ZL
;  call tapCharAsm


  pop r23
	pop ZL
	out _SFR_IO_ADDR(SREG),ZL
	pop ZL
	pop ZH
  reti

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; busy-wait 16-bit delay by Z cycles inc call.  
;;; Z MUST BE GREATER THAN 21, LESS THAN 65535!
;;; Resolution is 4 cycles
delay:
;;DEBUG DEBUG <<4 to slow things down
  push ZL   ; 2
  push ZH   ; 2
  sbiw ZL, 21  ; 5 (call) + 5 (ret) + 8 (p/p) + 4 (ror) - 1 (brne at exit)
bk1:
  lsr ZH      ; 1
  ror ZL      ; 1
  lsr ZH      ; 1
  ror ZL      ; 1
bk2:
  count1:     ; loop is 4 per iteration - 1 at exit
  sbiw ZL,1   ;2
  brne count1 ;1/2
bkcleanup:
  pop ZH ;2 
  pop ZL ;2
  ret

; Run through timetable once
run_timetable:
  push YH
  push YL
  push ZH
  push ZL
  push r26
  push r27

; pulseA1 to trigger scope
sbi _SFR_IO_ADDR(DDRA),PA3
sbi _SFR_IO_ADDR(DDRA),PA1
sbi _SFR_IO_ADDR(PORTA),PA1
nop
nop
nop
nop
nop
nop
nop
nop
nop
nop
nop
nop
cbi _SFR_IO_ADDR(PORTA),PA1

; DEBUG REMOVE? forcing low to start
sbi _SFR_IO_ADDR(PORTB),SYNC_PIN
sbi _SFR_IO_ADDR(PORTA),PA3
  
  ldi YH,hi8(timetable)
  ldi YL,lo8(timetable)
nexttime:
  ; check if last entry
  ldd r26,Y+0 ; flags
  cpi r26,0x80
  breq cleanup
  ldd r27,Y+1 ; count/repeat
repeat:
  ; toggle sync
  sbi _SFR_IO_ADDR(PINA),PA3
  sbi _SFR_IO_ADDR(PINB),SYNC_PIN

  ; delay 1
  ldd ZL, Y+2  ;lsb
  ldd ZH, Y+3
  call delay

  ; toggle sync
  sbi _SFR_IO_ADDR(PINA),PA3
  sbi _SFR_IO_ADDR(PINB),SYNC_PIN

  ; delay 2
  ldd ZL, Y+4  ;lsb
  ldd ZH, Y+5
  call delay

  dec r27
  brne repeat
endrepeat:

  adiw YL,6
  rjmp nexttime

cleanup:
;ldi r23,'Z'
;call tapCharAsm
  pop r27 
  pop r26 
  pop ZL
  pop ZH
  pop YL
  pop YH
  ret

render_bg_line:
  push r16
  push r17

  ldi r16,222
bg_line_loop:
  nop
  nop
  nop
  out _SFR_IO_ADDR(DATA_PORT),r16
  dec r16
  brne bg_line_loop
  
  pop r17
  pop r16
  ret

.macro jitter_fix9 tdiff
  ; Use the fact that brlo uses either 1 or 2 
  ; cycles based on comparison to un-jitter timing
	cpi \tdiff,1
	brlo .		;advance PC to next instruction
	cpi \tdiff,2
	brlo .		;advance PC to next instruction
	cpi \tdiff,3
	brlo .		;advance PC to next instruction
 	cpi \tdiff,4
	brlo .		;advance PC to next instruction
	cpi \tdiff,5
	brlo .		;advance PC to next instruction
	cpi \tdiff,6
	brlo .		;advance PC to next instruction
	cpi \tdiff,7
	brlo .		;advance PC to next instruction
	cpi \tdiff,8
	brlo .		;advance PC to next instruction
	cpi \tdiff,9
	brlo .		;advance PC to next instruction
.endm

.macro push_regs_r21_to_Z
  push YH
  push YL
  push ZH
	push ZL 
	;save flags & status register
	in ZL,_SFR_IO_ADDR(SREG)
  push ZL
  push XL
  push XH
  push r21
  push r22
  push r23
  push r24
.endm

.macro pop_regs_r21_to_Z
  pop r21
  pop r22
  pop r23
  pop r24
  pop XH
  pop XL
	pop ZL
	out _SFR_IO_ADDR(SREG),ZL
  pop ZL
  pop ZH
  pop YL
  pop YH
.endm

; Run through timetable entry
; triggered by interrupt
run_timetable_int:
  push_regs_r21_to_Z

nextloop_tti:
	lds ZL,_SFR_MEM_ADDR(TCNT1L) ; Read the timer time
	lds ZH,_SFR_MEM_ADDR(TCNT1H) 
  
  lds XL, tt_target_time+0 ; target timer time
  lds XH, tt_target_time+1
 
  ; Adjust for jitter/underestimation 
  ; we assume target > time!!  roll-over allowed!
  sub XH, ZH
  sbc XL, ZL

  ; is gross time adjustment needed?
  cp XL,0
  cpc XH,30
#if 0
  brsl small_adjust
  push ZL
  push ZH
  mov ZL, XL
  mov ZH, XH
  call delay
  pop LH
  pop ZL
#endif

small_adjust:
  ; fine jitter correction
  jitter_fix9 XL
 
  ; Now should be the target time + C 
 
  lds YL, tt_cur_ptr+0 ; Current pointer
  lds YH, tt_cur_ptr+1

  ; Do the deed - flip the signal!
  sbi _SFR_IO_ADDR(PINA),PA3
  sbi _SFR_IO_ADDR(PINB),SYNC_PIN

  ; Prepare for next 
  lds r22, tt_cur_stage
  dec r22
  brne doagain_tti ; Did I just finish second delay?

  ; Am I ready to move to next record?
  ldi r22, 2  ; reset stage
  sts tt_cur_stage, r22
  ; check repeat
  lds r23, tt_cur_repeat
  dec r23
  brne doagain_tti

  ; Move to next record
  adiw YL,6  ; next record
  sts tt_cur_ptr+0, YL
  sts tt_cur_ptr+1, YH
  ldd r23, Y+1 ; copy over repeat count
  sts tt_cur_repeat, r23

doagain_tti:
  lds XL, tt_target_time+0 ; target timer time
  lds XH, tt_target_time+1
  cpse r22, 2 ; cmp and skip if eq
  adiw YL,2 ; if 2nd stage, then get delay 2
  ldd r21, Y+2 ; load delay
  ldd r22, Y+3
  ;;; Is the delay large enough to justify returning from interrupt?
  cpi r22, 2
  brlo nextloop_tti
  
  add XH,YH    ; new target time
  adc XL,YL

  ; Set interrupt timer register 
 
  
cleanup_tti:
  pop_regs_r21_to_Z
  reti

